# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package Nick;

use strict;
use warnings FATAL => 'all';
use diagnostics;

require "db.pl";


my %name_to_id;
my %id_to_name;


# 0: nick name
# r: nick id
sub add($)
{
	my ($raw) = @_;
	my $id;
	my $query;

	my $name = &DB::safe_string($raw);

	&DB::transaction_start();
	$id = &DB::get_sequence("nicks_id_seq");
	$query = "INSERT INTO nicks (id, nick) VALUES ($id, '$name')";
	&DB::update($query);
	&DB::transaction_end();

	$name_to_id{$raw} = $id;
	$id_to_name{$id} = $raw;

	return $id;
}


# 0: nick name
# 1: audo add
# r: nick id
sub get_id($$)
{
	my ($raw, $autoadd) = @_;
	my ($query, $result);

	(defined($name_to_id{$raw})) && return $name_to_id{$raw};

	my $name = &DB::safe_string($raw);

	$query = "SELECT id, nick FROM nicks WHERE nick ILIKE '$name'";
	$result = &DB::query($query);
	if ($result->ntuples == 1) {
		my ($id, $t) = $result->fetchrow;
		$name_to_id{$raw} = $id;
		return $id;
	} elsif ($result->ntuples == 0) {
		if (defined($autoadd) && $autoadd == 1) {
			return &add_nick($name);
		}
		return;
	}

	# more than one entry? PostgreSQL/ILIKE seems to handle '_' as a
	# wildcard... Fine, we'll manually pick the correct one.
	$name =~ tr/A-Z/a-z/;
	while (my ($id, $str) = $result->fetchrow) {
		$str =~ tr/A-Z/a-z/;
		if ($str eq $name) {
			$name_to_id{$raw} = $id;
			return $id;
		}
	}
	return;
}


# 0: id
# ret: nick name
sub get_name($)
{
	my ($raw) = @_;
	my ($query, $result);

	(defined($id_to_name{$raw})) && return $id_to_name{$raw};
	
	my $id = &DB::safe_string($raw);

	$query = "SELECT nick FROM nicks WHERE id = $id";
	$result = &DB::query($query);
	($result->ntuples == 0) && return;

	my ($name) = $result->fetchrow;
	$id_to_name{$raw} = $name;

	return $name;
}


# ...: nick id
# r: hash id->nick
sub get_names(@)
{
	my @ids = @_;
	my ($query, $result);

	$query = "SELECT id, nick FROM nicks WHERE ";
	foreach my $raw (@ids) {
		my $id = &DB::safe_string($raw);
		$query .= "id = $id OR "
	}
	$query =~ s/OR $//;

	$result = &DB::query($query);
	my %names;
	while (my ($id, $name) = $result->fetchrow) {
		$names{$id} = $name;
	}

	return %names;
}


return 1;
